/*
 * QEMU Error Objects
 *
 * Copyright IBM, Corp. 2011
 *
 * Authors:
 *  Anthony Liguori   <aliguori@us.ibm.com>
 *
 * This work is licensed under the terms of the GNU LGPL, version 2.  See
 * the COPYING.LIB file in the top-level directory.
 */

#include "qemu-common.h"
#include "error.h"
#include "qjson.h"
#include "qdict.h"
#include "qapi-types.h"
#include "qerror.h"
#include <string.h>

struct Error
{
    char *msg;
    ErrorClass err_class;
};

void error_set(Error **errp, ErrorClass err_class, const char *fmt, ...)
{
    Error *err;
    va_list ap;

    if (errp == NULL) {
        return;
    }
    assert(*errp == NULL);

    err = g_malloc0(sizeof(*err));

    va_start(ap, fmt);
    err->msg = g_strdup_vprintf(fmt, ap);
    va_end(ap);
    err->err_class = err_class;

    *errp = err;
}

Error *error_copy(const Error *err)
{
    Error *err_new;

    err_new = g_malloc0(sizeof(*err));
    err_new->msg = g_strdup(err->msg);
    err_new->err_class = err->err_class;

    return err_new;
}

/* RHEL-6 note:
 *
 * The following function, error_vsetg_errno(), and the implementation of
 * error_setg_errno() and error_setg() below, are RHEL-6 only compatibility
 * code. The RHEL-6 Error object is incompatible with that of upstream, but the
 * structure is not externally visible.
 */
static void error_vsetg_errno(Error **errp, int os_errno, const char *fmt,
                              va_list ap)
{
    char *msg;

    msg = g_strdup_vprintf(fmt, ap);
    if (os_errno != 0) {
        char *msg2;

        msg2 = g_strdup_printf("%s: %s", msg, strerror(os_errno));
        free(msg);
        msg = msg2;
    }
    if (errp != NULL & *errp != NULL)
        error_set(errp, error_get_class(*errp), QERR_GENERIC_ERROR, msg);
    free(msg);
}

void error_setg_errno(Error **errp, int os_errno, const char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    error_vsetg_errno(errp, os_errno, fmt, ap);
    va_end(ap);
}

void error_setg(Error **errp, const char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    error_vsetg_errno(errp, 0, fmt, ap);
    va_end(ap);
}

bool error_is_set(Error **errp)
{
    return (errp && *errp);
}

ErrorClass error_get_class(const Error *err)
{
    return err->err_class;
}

const char *error_get_pretty(Error *err)
{
    return err->msg;
}

void error_free(Error *err)
{
    if (err) {
        g_free(err->msg);
        g_free(err);
    }
}

void error_propagate(Error **dst_err, Error *local_err)
{
    if (dst_err && !*dst_err) {
        *dst_err = local_err;
    } else if (local_err) {
        error_free(local_err);
    }
}
